# -*- coding: utf-8 -*-
from zope.viewlet.viewlet import ViewletBase

from AccessControl import getSecurityManager
from Products.CMFCore.utils import getToolByName

class CatalogViewlet(ViewletBase):

    def update(self):

        self.indices = []
        self.metadata = []

        ct = getToolByName(self.context, 'portal_catalog')

        # get the catalog record (brain) of the object
        path = '/'.join(self.context.getPhysicalPath())
        result = ct.searchResults(path={'query': path, 'depth': 0})

        if result and len(result) > 0:
            brain = result[0]
            for key, value in ct.getIndexDataForRID(brain.getRID()).items():
                self.indices.append({'name': key, 'value': value})
            self.indices.sort(key=lambda k: k['name'])
            for key, value in ct.getMetadataForRID(brain.getRID()).items():
                self.metadata.append({'name': key, 'value': value})
            self.metadata.sort(key=lambda k: k['name'])
        else:
            brain = None

        self.catalogStatus = brain



# -*- coding: utf-8 -*-
from zope.publisher.interfaces import IView
from zope.viewlet.viewlet import ViewletBase
from zope.pagetemplate.interfaces import IPageTemplateSubclassing

class PublishedViewlet(ViewletBase):

    def update(self):
        self.published = self.request.get('PUBLISHED', None)
        
        self.name = getattr(self.published, '__name__', None)
        if self.name is None:
            self.name = getattr(self.published, 'id', None)
        
        self.cls = self.published.__class__

        self.filename = None

        if IPageTemplateSubclassing.providedBy(self.published):
            self.filename = self.published.pt_source_file()
        
        if IView.providedBy(self.published):
            # Look for a page template
            for attr in ('index', 'template', '__call__'):
                pt = getattr(self.published, attr, None)
                if hasattr(pt, 'filename'):
                    self.filename = pt.filename
                    break



# -*- coding: utf-8 -*-
from zope.interface import Interface
from zope.viewlet.interfaces import IViewletManager

class IDebugToolbarLayer(Interface):
    """Browser layer marker interface
    """

class IDebugToolbarViewletManager(IViewletManager):
    """Marker interface for the debug toolbar viewlet manager
    """



# -*- coding: utf-8 -*-
from zope.viewlet.viewlet import ViewletBase
from zc.lockfile import LockError

class GlobalViewlet(ViewletBase):

    def update(self):
        from App.config import getConfiguration
        self.debug = getConfiguration().debug_mode

        import App.config
        config = App.config.getConfiguration()

        self.config = []
        for name in sorted(config.getSectionAttributes()):

            if name in (
                'access', 'databases', 'eventlog', 'servers',
            ):
                continue

            self.config.append((name, getattr(config, name)))

        # BBB: import for Zope2
        try:
            self.servers = config.servers
        except AttributeError:
            self.servers = []

        # BBB: import for Zope2
        try:
            self.appInfo = self.context.getPhysicalRoot()['Control_Panel']
        except KeyError:
            self.appInfo = self.context.getPhysicalRoot().Control_Panel

        self.databases = []
        paths = dict([(x[1], x[0],) for x in config.dbtab.mount_paths.items()])

        for db in config.databases:
            name = db.name
            # BBB: import for Zope2
            try:
                real_db = self.appInfo.Database[name]
                dbtabEntry = config.dbtab.databases[name]

                self.databases.append({
                    'name': name,
                    'location': real_db.db_name(),
                    'size': real_db.db_size(),
                    'cacheSize': dbtabEntry.getCacheSize(),
                    'mount': paths.get(name, None),
                })
            except LockError:
                # lock error when trying to access database
                # https://github.com/zopefoundation/Zope/issues/360
                db_config = db.config
                self.databases.append({
                    'name': name,
                    'location': 'TODO could not access db',
                    'size': 'TODO could not access db',
                    'cacheSize': db_config.cache_size,
                    'mount': paths.get(name, None),
                })



# -*- coding: utf-8 -*-
from plone.app.standardtiles.common import ProxyViewletTile


class ToolbarTile(ProxyViewletTile):
    """A footer tile."""
    manager = 'plone.portalfooter'
    viewlet = 'plone.app.debugtoolbar.toolbar'



# -*- coding: utf-8 -*-
import pkg_resources
from zope.viewlet.viewlet import ViewletBase
from Products.CMFCore.utils import getToolByName

class VersionsViewlet(ViewletBase):

    def update(self):
        packages = []
        self.ploneVersion = None

        for distribution in pkg_resources.working_set:
            name = distribution.project_name
            packages.append({
                'name': name,
                'version':distribution.version,
            })

            if name.lower() == 'plone':
                self.ploneVersion = distribution.version
            elif self.ploneVersion is None and name.lower() == 'products.cmfplone':
                self.ploneVersion = distribution.version
            
        self.packages = sorted(packages, key=lambda x: x['name'].lower())

        mt = getToolByName(self.context, 'portal_migration')
        self.needUpgrading = mt.needUpgrading()


# -*- coding: utf-8 -*-
from zope.viewlet.viewlet import ViewletBase

class RequestViewlet(ViewletBase):

    def update(self):
        self.headers = []
        self.environ = []

        for k in sorted(self.request.environ):
            if k.upper().startswith('HTTP_'):
                header = '-'.join([x.capitalize() for x in k[5:].split('_')])
                self.headers.append((header, self.request.environ[k],))
            else:
                self.environ.append((k, self.request.environ[k],))



# -*- coding: utf-8 -*-
from zope.viewlet.viewlet import ViewletBase

from AccessControl import getSecurityManager
from Products.CMFCore.utils import getToolByName

class WorkflowViewlet(ViewletBase):

    def update(self):
        
        # Workflow
        workflow = getToolByName(self.context, 'portal_workflow')

        self.workflowStatus = workflow.getInfoFor(self.context, 'review_state', None)

        workflows = workflow.getWorkflowsFor(self.context)

        self.workflowStates = [
            workflow.getInfoFor(self.context, wf.state_var, None)
            for wf in workflows
        ]

        self.workflowNames = [wf.getId() for wf in workflows]

        # Permissions

        self.roles = self.context.validRoles()
        self.permissionInfo = []

        sm = getSecurityManager()

        for p in self.context.permission_settings():
            name = p['name']
            acquired = p['acquire'] == 'CHECKED'
            granted = bool(sm.checkPermission(name, self.context))

            roles = [r['selected'] == 'SELECTED'
                        for r in self.context.rolesOfPermission(name)]
            
            self.permissionInfo.append({
                'name': name,
                'granted': granted,
                'acquired': acquired,
                'roles': roles,
            })



# -*- coding: utf-8 -*-
from AccessControl import getSecurityManager
from zope.viewlet.viewlet import ViewletBase

class UserViewlet(ViewletBase):

    def update(self):
        securityManager = getSecurityManager()

        self.user = securityManager.getUser()
        try:
            self.fullname = self.user.getProperty('fullname', None)
        except AttributeError:
            self.fullname = ""



# -*- coding: utf-8 -*-
from plone.app.debugtoolbar.testing import PLONE_APP_DEBUGTOOLBAR_INTEGRATION_TESTING  # noqa: E501
from Products.CMFPlone.utils import get_installer

import unittest


class TestSetup(unittest.TestCase):

    layer = PLONE_APP_DEBUGTOOLBAR_INTEGRATION_TESTING

    def setUp(self):
        self.app = self.layer['app']
        self.portal = self.layer['portal']
        self.request = self.layer['request']
        self.qi_tool = get_installer(self.portal, self.request)

    def test_product_is_installed(self):
        """ Validate that our products GS profile has been run and the product
            installed
        """
        pid = 'plone.app.debugtoolbar'
        self.assertTrue(
            self.qi_tool.is_product_installed(pid),
            'package appears not to have been installed',
        )



# -*- coding: utf-8 -*-
import re
import threading
import traceback
import six

from zope.component import queryMultiAdapter
from zope.publisher.browser import BrowserView
from zope.viewlet.viewlet import ViewletBase

from paste.evalexception import evalcontext
from paste.exceptions import formatter

from AccessControl import Unauthorized
from AccessControl import getSecurityManager
from Products.CMFCore.utils import getToolByName
from Products.PageTemplates.Expressions import getEngine
from Products.CMFPlone.PloneBaseTool import createExprContext


try:
    from html import escape
except ImportError:
    from cgi import escape


class Variables(object):
    """Store local variables. Allow one set of variables per user id, and
    invalidate if the path of the context changes.
    """

    def __init__(self):
        self._lock = threading.RLock()
        self._storage = {}

    def invalidate(self, context, force=False):
        with self._lock:
            userId = getSecurityManager().getUser().getId()
            if userId not in self._storage:
                return

            if force or self._storage[userId]['path'] != '/'.join(context.getPhysicalPath()):
                del self._storage[userId]

    def get(self, context):
        self.invalidate(context)

        with self._lock:
            userId = getSecurityManager().getUser().getId()
            return self._storage.get(userId, {}).get('vars', {})

    def update(self, context, variables):
        with self._lock:
            userId = getSecurityManager().getUser().getId()

            self._storage[userId] = {
                'path': '/'.join(context.getPhysicalPath()),
                'vars': variables,
            }

VARS = Variables()

def htmlQuote(v):
    # Borrowed from Paste
    if v is None:
        return ''
    return escape(str(v), 1)


def preserveWhitespace(v, quote=True):
    # Borrowed from Paste
    if quote:
        v = htmlQuote(v)

    def _repl_nbsp(match):
        if len(match.group(2)) == 1:
            return '&nbsp;'
        return match.group(1) + '&nbsp;' * (len(match.group(2))-1) + ' '

    v = v.replace('\n', '<br>\n')
    v = re.sub(r'()(  +)', _repl_nbsp, v)
    v = re.sub(r'(\n)( +)', _repl_nbsp, v)
    v = re.sub(r'^()( +)', _repl_nbsp, v)
    return '<code>%s</code>' % v

class InteractiveViewlet(ViewletBase):

    def update(self):
        VARS.invalidate(self.context, force=True)

class InteractiveResponse(BrowserView):

    def __call__(self):

        if self.request.method != 'POST':
            raise Unauthorized()

        line = self.request.form.get('line', '')
        if not line.strip():
            return ''

        line = line.rstrip() + '\n'

        portal = getToolByName(self.context, 'portal_url').getPortalObject()
        app = portal.getPhysicalRoot()

        globs = {
            'context': self.context,
            'request': self.request,
            'portal': portal,
            'app': app,
        }

        globs.update(VARS.get(self.context))
        context = evalcontext.EvalContext({}, globs)
        output = context.exec_expr(line)
        VARS.update(self.context, globs)

        line_html = formatter.str2html(line)
        return ('<code style="color: #060">&gt;&gt;&gt;</code> '
                '%s<br />\n%s'
                % (preserveWhitespace(line_html, quote=False),
                   preserveWhitespace(output)))

class TALESResponse(BrowserView):

    def __call__(self):

        if self.request.method != 'POST':
            raise Unauthorized()

        line = self.request.form.get('line', '')
        if not line.strip():
            return ''

        line = line.rstrip() + '\n'

        expr = self.compileExpression(line)
        try:
            output = expr(self.createExpressionContext())
        except:
            output = "%s" % traceback.format_exc()

        if isinstance(output, six.text_type):
            output = output.encode('ascii', 'xmlcharrefreplace')
        elif not isinstance(output, str):
            output = repr(output)

        return preserveWhitespace(output)


    def createExpressionContext(self):
        """Using the Plone expression context with all its variables
        """
        context_state = queryMultiAdapter((self.context, self.request), name=u'plone_context_state')
        portal_state = queryMultiAdapter((self.context, self.request), name=u'plone_portal_state')
        return createExprContext(context_state.folder(), portal_state.portal(), self.context)

    def compileExpression(self, text):
        return getEngine().compile(text.strip())



# -*- coding: utf-8 -*-
import json

from zope.interface import Interface
from zope.interface import implementer
from zope.component import adapts
from zope.annotation.interfaces import IAnnotations

from plone.transformchain.interfaces import ITransform

from plone.app.debugtoolbar.browser.interfaces import IDebugToolbarLayer

def delay(request, name, fn):
    """Register a function that will be called at the end of the request.
    """

    ann = IAnnotations(request, None)
    if ann is None:
        return
    ann.setdefault('plone.app.debugtoolbar.delayed', {})[name] = fn

@implementer(ITransform)
class DelayedWriteTransformer(object):
    adapts(Interface, IDebugToolbarLayer)

    order = 9999

    def __init__(self, published, request):
        self.published = published
        self.request = request
    
    def transformUnicode(self, result, encoding):
        self.saveCookie()
        return None
    
    def transformBytes(self, result, encoding):
        self.saveCookie()
        return None

    def transformIterable(self, result, encoding):
        self.saveCookie()
        return None
    
    def saveCookie(self):
        data = self.getData()
        if data is not None:
            self.request.response.setCookie('plone.app.debugtoolbar', data, quoted=False, path='/')
        
    def getData(self):
        data = {}
        ann = IAnnotations(self.request, None)
        if ann is None:
            return data
        
        delayed = ann.get('plone.app.debugtoolbar.delayed', {})
        if not delayed:
            return None
        
        for key, fn in delayed.items():
            data[key] = fn(self.request)
        
        return json.dumps(data)



# -*- coding: utf-8 -*-
from zope.viewlet.viewlet import ViewletBase
from plone.app.debugtoolbar.delayedwrite import delay

def captureHeaders(request):
    headers = []

    for k in sorted(request.response.headers):
        header = '-'.join([x.capitalize() for x in k.split('-')])
        headers.append((header, request.response.headers[k],))
    
    return headers

def captureStatus(request):
    return request.response.status

class ResponseViewlet(ViewletBase):

    def update(self):
        delay(self.request, 'response_headers', captureHeaders)
        delay(self.request, 'response_status', captureStatus)



# -*- coding: utf-8 -*-
from zope.interface import directlyProvidedBy
from zope.component import queryUtility
from zope.viewlet.viewlet import ViewletBase
from zope.publisher.interfaces.browser import IBrowserSkinType

from Products.CMFCore.utils import getToolByName


class ThemeViewlet(ViewletBase):

    def update(self):

        skins = getToolByName(self.context, 'portal_skins')
        url = getToolByName(self.context, 'portal_url')

        defaultSkin = skins.getDefaultSkin()
        requestVariable = skins.getRequestVarname()

        self.portal_url = url()
        self.portal = url.getPortalObject()

        # CMF skin
        self.themeName = self.request.get(requestVariable, defaultSkin)
        self.skinPaths = dict(skins.getSkinPaths()).get(self.themeName, '').split(',')  # noqa: E501

        # Browser layers
        self.themeLayer = queryUtility(IBrowserSkinType, name=self.themeName)
        self.layers = list(directlyProvidedBy(self.request).flattened())

        # CSS/JS composition
        self.css = getToolByName(self.context, 'portal_css', None)
        self.js = getToolByName(self.context, 'portal_javascripts', None)



# -*- coding: utf-8 -*-
import types
import inspect
import six

from zope.interface import Interface
from zope.interface import providedBy, directlyProvidedBy
from zope.component import getAdapters
from zope.publisher.interfaces import IView
from zope.viewlet.viewlet import ViewletBase

from Acquisition import aq_base
from Products.CMFCore.interfaces import IDynamicType
from Products.CMFDynamicViewFTI.interfaces import IDynamicViewTypeInformation
from Products.Five.browser.metaconfigure import ViewMixinForTemplates

class ContextViewlet(ViewletBase):

    def update(self):

        self.path = '/'.join(self.context.getPhysicalPath())
        self.cls = self.context.__class__

        self.fti = None
        self.methodAliases = None

        if IDynamicType.providedBy(self.context):
            self.fti = self.context.getTypeInfo()
            self.methodAliases = sorted(self.fti.getMethodAliases().items())

        self.defaultView = None
        self.viewMethods = []
        if IDynamicViewTypeInformation.providedBy(self.fti):
            self.defaultView = self.fti.defaultView(self.context)
            self.viewMethods = self.fti.getAvailableViewMethods(self.context)

        directly_provided = directlyProvidedBy(self.context)
        self.provided = list(providedBy(self.context).flattened())
        self.provided.sort(key=lambda i: i.__identifier__)
        self.provided = ({'dottedname': i.__identifier__,
                          'is_marker': i in directly_provided}
                          for i in self.provided)
        self.views = []

        generator = getAdapters((self.context, self.request,), Interface)
        while True:
            try:
                name, view = next(generator)

                if not IView.providedBy(view):
                    continue

                cls = view.__class__
                module = cls.__module__
                template = None

                if isinstance(view, ViewMixinForTemplates):
                    template = view.index.filename
                else:
                    for attr in ('index', 'template', '__call__'):
                        pt = getattr(view, attr, None)
                        if hasattr(pt, 'filename'):
                            template = pt.filename
                            break

                # Deal with silly Five metaclasses
                if (
                    module == 'Products.Five.metaclass' and
                    len(cls.__bases__) > 0
                ):
                    cls = cls.__bases__[0]
                elif cls == ViewMixinForTemplates:
                    cls = None

                self.views.append({
                    'name': name,
                    'class': cls,
                    'template': template,
                })
            except StopIteration:
                break
            except:
                # Some adapters don't initialise cleanly
                pass

        self.views.sort(key=lambda v: v['name'])

        self.methods = []
        self.variables = []

        _marker = object()
        for name in sorted(dir(aq_base(self.context))):
            attr = getattr(aq_base(self.context), name, _marker)
            if attr is _marker:
                continue

            # FIXME: Should we include ComputedAttribute here ? [glenfant]
            if isinstance(attr, (int, float, six.string_types, bool, list, tuple, dict, set, frozenset)):
                self.variables.append({
                    'name': name,
                    'primitive': True,
                    'value': attr,
                })
            elif (
                isinstance(attr, (types.MethodType, types.BuiltinFunctionType, types.BuiltinMethodType, types.FunctionType)) or
                attr.__class__.__name__ == 'method-wrapper',
            ):

                source = None
                if name.endswith('__roles__'):
                    # name without '__roles__' is the last in self.methods since we're in a sorted(...) loop
                    if callable(attr):
                        secu_infos = attr()
                    else:
                        secu_infos = attr
                    if secu_infos is None:
                        secu_label = 'Public'
                    else:
                        secu_label = ''
                        try:
                            secu_label += 'Roles: ' + ', '.join([r for r in secu_infos[:-1]])
                        except TypeError:
                            # Avoid "TypeError: sequence index must be
                            # integer, not 'slice'", which occurs with the
                            # ``C`` security implementation. This is a rare
                            # case. In development you normally use the
                            # ``Python`` security implementation, where this
                            # error doesn't occur.
                            pass
                        secu_label += '. Permission: ' + secu_infos[-1][1:-11]  # _x_Permission -> x
                    self.methods[-1]['secu_infos'] = secu_label
                else:
                    try:
                        source = inspect.getsourcefile(attr)
                    except TypeError:
                        None

                    signature = name + "()"
                    try:
                        signature = name + inspect.formatargspec(*inspect.getargspec(attr))
                    except TypeError:
                        pass

                    self.methods.append({
                        'signature': signature,
                        'filename': source,
                        'help': inspect.getdoc(attr),
                    })
            else:
                self.variables.append({
                    'name': name,
                    'primitive': False,
                    'value': str(attr),
                })



# -*- coding: utf-8 -*-
from Acquisition import aq_parent
from zope.component import getMultiAdapter
from zope.viewlet.viewlet import ViewletBase

try:
    import plone.reload
    RELOAD_INSTALLED = True
except:
    RELOAD_INSTALLED = False


class ReloadViewlet(ViewletBase):
    reload_installed = RELOAD_INSTALLED

    def update(self):
        portal_state = getMultiAdapter((self.context, self.request),
                                       name="plone_portal_state")
        portal = portal_state.portal()
        app = aq_parent(portal)
        self.code_url = '%s/@@reload?action=code' % app.absolute_url()
        self.zcml_url = '%s/@@reload?action=zcml' % app.absolute_url()

        self.css_url = '%s/portal_css/cookResources' % (
            portal.absolute_url())
        self.js_url = '%s/portal_javascripts/cookResources' % (
            portal.absolute_url())



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages

version = '1.2.3'

long_description = (
    open('README.rst').read()
    + '\n' +
    'Contributors\n'
    '============\n'
    + '\n' +
    open('CONTRIBUTORS.rst').read()
    + '\n' +
    open('CHANGES.rst').read()
    + '\n')

setup(
    name='plone.app.debugtoolbar',
    version=version,
    description="Debug toolbar for Plone",
    long_description=long_description,
    # Get more strings from
    # https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Plone",
        "Framework :: Plone :: 5.1",
        "Framework :: Plone :: 5.2",
        "Framework :: Plone :: 6.0",
        "License :: OSI Approved :: GNU General Public License (GPL)",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    keywords='plone debug toolbar',
    author='Martin Aspeli',
    author_email='optilude@gmail.com',
    url='https://pypi.org/project/plone.app.debugtoolbar/',
    license='GPL',
    packages=find_packages('src'),
    package_dir={'': 'src'},
    namespace_packages=['plone', 'plone.app'],
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'setuptools',
        'Products.CMFPlone',
        'zope.annotation',
        'plone.transformchain',
        'Paste',
        'six'
    ],
    extras_require={'test': ['plone.app.testing']},
    entry_points="""
    # -*- Entry points: -*-
    [z3c.autoinclude.plugin]
    target = plone
    """,
    )



# -*- coding: utf-8 -*-
from plone.app.testing import applyProfile
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer


class PloneAppDebugtoolbar(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE, )

    def setUpZope(self, app, configurationContext):
        # Load ZCML for this package
        import plone.app.debugtoolbar
        self.loadZCML(package=plone.app.debugtoolbar)

    def setUpPloneSite(self, portal):
        applyProfile(portal, 'plone.app.debugtoolbar:default')


PLONE_APP_DEBUGTOOLBAR_FIXTURE = PloneAppDebugtoolbar()
PLONE_APP_DEBUGTOOLBAR_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_DEBUGTOOLBAR_FIXTURE, ),
    name="PloneAppDebugtoolbar:Integration",
)



# -*- coding: utf-8 -*-
from Products.CMFCore.utils import getToolByName

import logging


logger = logging.getLogger('plone.app.debugtoolbar upgrade')


def upgrade_1_to_2(context):
    """Remove JS and CSS resources from portal_css and portal_js registry.
    """
    def unregister_resource(registry, resource):
        if registry and registry.getResource(resource):
            registry.unregisterResource(resource)
            logger.info("Removed {0} from {1}".format(resource, registry.id))

    # Unregister JavaScript
    unregister_resource(
        getToolByName(context, 'portal_javascripts'),
        '++resource++plone.app.debugtoolbar/debugtoolbar.js'
    )

    # Unregister CSS
    unregister_resource(
        getToolByName(context, 'portal_css'),
        '++resource++plone.app.debugtoolbar/debugtoolbar.css'
    )
